// AYUCR Camera Controller

include "..\\PocketGUI\\PocketGuiLib.pc"
//include "PocketGui.h"
include "SerialLib.h"

#define VERSION "2.3"

//------------------------------------------------------------------------
@cid	"AYUc";
@name	"AYUCR Camera Controller";
@dbname	"AYUCR Camera Controller";
@ver	VERSION;
@licon1	"large.bmp";
@sicon1 "small.bmp";

//------------------------------------------------------------------------

#define PROGLEN 16
#define PROGUSED 9
int proglen = 8; // default to old size
int prog[PROGLEN];
string recvstr;

//------------------------------------------------------------------------

#define PROGBUT 40
#define GETBUT	41
#define SAVEBUT 42
#define LOADBUT 43
#define TESTBUT 44
#define MODBUT	45
#define VERBUT	46
#define SIZEBUT	47

create_controls()
{
	int i=1;
	GCount (i++,"",21,0,5);
	GCount (i++,"",21,2,5);
	GCount (i++,"",21,4,5);
	GCount (i++,"",21,6,5);
	GCount (i++,"",21,8,5);
	GCount (i++,"",21,10,5);
	GCount (i++,"",21,12,5);
	GCount (i++,"",21,14,5);
	GCount (i++,"",21,16,5);
	GCount (i++,"",21,18,5);

	GLabel(i++,"Launch Detect (1/100 sec)",1,0,20);
	GLabel(i++,"Launch Delay (1/10 sec)",1,2,20);
	GLabel(i++,"Number of Groups",1,4,20);
	GLabel(i++,"Pictures per Group",1,6,20);
	GLabel(i++,"Picture Interval (1/10 sec)",1,8,20);
	GLabel(i++,"Group Interval (sec)",1,10,20);
	GLabel(i++,"Shutter Speed (1/10 sec)",1,12,20);
	GLabel(i++,"",1,14,20);
	GLabel(i++,"",1,16,20);
	GLabel(i++,"",1,18,20);

	GText(MODBUT,"",1,19,4);
	GText(VERBUT,"",4,19,6);
	GText(SIZEBUT,"",9,19,3);

	GButton (PROGBUT,"Prog",1,22,5);
	GButton (GETBUT,"Read",6,22,5);
	GButton (LOADBUT,"Load",11,22,5);
	GButton (SAVEBUT,"Save",16,22,5);
	GButton (TESTBUT,"Test",21,22,5);
}

//------------------------------------------------------------------------

save_program(string name)
{
    int i;

    dbcreate(name);

    for(i=0; i<PROGLEN; i++)
        dbwrite(prog[i]);

    dbclose();
}

load_program(string name)
{
    int i;

    dbopen(name);

    for(i=0; i<PROGLEN; i++)
        prog[i] = dbread('i');

    dbclose();
}

read_fields()
{
     int i;

     for(i=0; i<PROGUSED; i++)
         prog[i] = GState(i+1);
}

load_fields()
{
     int i;
     string s;

    for(i=0; i<PROGUSED; i++) {
        GSelect (i+1, prog[i]);
    }
}

send_program ()
{
    int i;
    char ack[3];

    read_fields();

    seropen (2400, "8N1X",1000);

    tone (1000,25);
    tone (500,50);

    sersendstr ("/P");
    sleep (600);

    sersendarray(prog, proglen);
    serrecvarray(ack,3);
    serclose ();

    if (ctostr (ack) == "OK")
        alert ("Programming sucessful");
    else
        alert ("Programming failed -" + ctostr (ack));
}

recv_program ()
{
    int i, c;
    char ack[3];

    seropen (2400, "8N1X",100);

    tone (500,50);
    tone (1000, 25);

    // Send the dump command
    sersendstr ("/D");

    for (i = 0; i < proglen; i++) {
        c=serrecv();
        if(c>255)
            break;
       
        prog[i]=c;
    }

    serrecvarray(ack,3);
    serclose();

    if (i<proglen || ctostr (ack) != "OK"){
        alert ("Dump failed reading " + i + " bytes");
        read_fields();
        return;
     }

    load_fields();
}

test_shutter ()
{
    char ack[3];
    seropen (2400, "8N1X",500);

    // Send the command
    sersendstr ("/S");

    serrecvarray(ack,3);
    serclose();

    if(ctostr (ack) != "OK"){
        alert ("Comm Error");
    }
}

get_model_version()
{
    int i, c;
    char str[12];
    string ver, mod;

    seropen (2400, "8N1X", 50);

	// Send the model command
	sersendstr ("/M");
    serrecvarray(str,12);
    
    mod = ctostr (str);
    
    for (i=0; i < strlen (mod);i++){
        if(mod@[i] > ' '){
        	mod = substr(mod, i, 3);
            GSet(MODBUT, mod);
            break;
        }
    }
	
	if (mod == "ACC") {
        GSet(18, "Servo Mode");
        GSet(19, "Keepalive Interval (10 sec)");
        GSet(20, "");
    }
    else if (mod == "APC") {
        GSet(18, "Continuous Mode");
        GSet(19, "Poweron Time");
        GSet(20, "Cycle Count");
    }
		
	
    // Send the version command
    sersendstr ("/V");
    serrecvarray(str,12);

    ver = ctostr (str);
    for(i=0; i < strlen (ver); i++){
        if(ver@[i] == 'V'){
            GSet(VERBUT, substr(ver,i,5));
            return;
        }
    }

    GSet(VERBUT, "V?.??");

    serclose();
}

get_proglen()
{
    char str[12];
    string size;

    seropen (2400, "8N1X",50);

    // Send the command
    sersendstr("/Z");

    serrecvarray(str,12);
    serclose();

    size=ctostr(str);
    if(strright(size,2)=="OK") {
        proglen=str[0];
    }
    
    GSet(SIZEBUT, proglen);
}

main()
{
	int e;

    GInit("AYUCR Camera Controller " + VERSION);
    create_controls();

    load_program("AYUCR");
    load_fields();

     // Get version and set proglen
    get_model_version();
    get_proglen();

    GRefresh();

    do{
        e = GEvent();

        if(e == PROGBUT) {
            send_program();
        }
        else if(e == GETBUT) {
            recv_program();
        }
        else if(e == SAVEBUT) {
           read_fields();
           save_program("AYUCR");
           tone (1000, 25);
       }
       else if(e == LOADBUT) {
           load_program("AYUCR");
           load_fields();
           tone(1500, 25);
       }
       else if(e == TESTBUT) {
           tone(1500, 25);
           test_shutter();
       }
    } while (1);
}
